class Processor
  constructor: (selector) ->
    @root = $ selector
    @columnLabels = ['', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
      'T',
      'U', 'V', 'W', 'X', 'Y', 'Z']
    @rowLabels = [0...100]
    @selected = $ ''
    @buttons = [
      ['help', 'Nápověda', @showHelp]
      ['bold', 'Tučně', => @toggleClass 'b']
      ['italic', 'Kurzíva', => @toggleClass 'i'],
      ['underline', 'Podtržené', => @toggleClass 'u'],
      ['border', 'Ohraničení', => @toggleClass 'border'],
      ['size-plus', 'A+', => @sizePlus()],
      ['size-minus', 'A-', => @sizeMinus()],
      ['remove-format', 'Deformátovat', => @deformat()],
      ['delete-row', 'X řádek', => @deleteRow()],
      ['delete-column', 'X sloupec', => @deleteColumn()],
    ]
    @selects = [
      ['font', [
        ['f1', 'Georgia'],
        ['f2', 'Times New Roman'],
        ['f3', 'Trebuchet MS'],
        ['f4', 'Courier New'],
        ['f-ugly', 'Comic Sans MS'],
      ], (e) => @changeFont(e)]
    ]

    @colors = [
      ['color', 'red black green yellow blue'.split(' '), 'písmo', (e) => @changeColor(e, 'color')],
      ['background-color', 'red black green yellow blue'.split(' '), 'pozadí',
        (e) => @changeColor(e, 'background-color')],
    ]


  init: ->
    @buildUi()
    @initEditable()
    @initFastEditation()

  initEditable: ->
    @body.on 'click', 'td input', (e) =>
      @selected = $ e.target
      @body.find('.focus').removeClass 'focus'
      @selected.addClass 'focus'
      @edit.val @selected.val()
      @edit.focus()
      @toolbar.find('.font').val(@selected.data('font') || 'f1')

    @edit.on 'input', (e) =>
      $this = $ e.target
      @selected.val $this.val()
      @selected.attr 'value', $this.val()
      @recompute()

  buildUi: ->
    @toolbar = $('<div />',
      class: 'toolbar'
    ).appendTo(@root)
    @edit = $('<input />',
      class: 'edit',
      type: 'text'
    ).appendTo(@root)
    @table = $('<table />').appendTo(@root)
    @head = $('<thead />').appendTo(@table)
    @body = $('<tbody />').appendTo(@table)

    @buildCells(@body, @head)
    @buildToolbar(@toolbar)

  buildCells: (body, head) ->
# heading
    headRow = $ '<tr />'
    for column, x in @columnLabels
      $('<th />').text(column).appendTo headRow
    head.append headRow

    # inner table
    for row, y in @rowLabels
      row = $ '<tr />'
      $('<th />').text(y + 1).appendTo row
      for column, x in @columnLabels
        $('<td />',
          'data-x': x,
          'data-y': y
        ).html($('<input type="text" tabindex="0"/>')).appendTo row
      body.append row


  buildToolbar: (toolbar) ->
    for button, i in @buttons
      $('<button />').addClass(button[0]).text(button[1]).appendTo(toolbar).click(button[2])

    for select, i in @selects
      selectEl = $('<select />').addClass(select[0]).appendTo(toolbar).on 'change', select[2]
      for option, _ in select[1]
        $('<option />',
          value: option[0]
        ).text(option[1]).appendTo(selectEl)

    for color, i in @colors
      input = $('<input />',
        list: color[0],
        class: color[0],
        type: 'text',
        placeholder: color[2]
      ).appendTo(toolbar).on 'input', color[3]

      input.on 'focus', ->
        input.val('')

      datalist = $ '<datalist />',
        id: color[0]

      for option, _ in color[1]
        $('<option />',
          value: option
        ).appendTo(datalist)

      datalist.appendTo(@root)

  showHelp: ->
    window.open('help.html')

  sizePlus: ->
    size = parseInt(@selected.css('font-size')) + 2
    @selected.css 'font-size', size
    @edit.focus()

  sizeMinus: ->
    size = parseInt(@selected.css('font-size')) - 2
    @selected.css 'font-size', size
    @edit.focus()

  changeFont: (e) ->
    val = $(e.target).val()
    @selected.toggleClass(val).data('font', val)
    @edit.focus()

  changeColor: (e, property) ->
    color = $(e.target).val()
    if @isValidCssColor(color)
      @selected.css(property, color)
      @edit.focus()

  deformat: () ->
    @selected.removeClass().addClass 'focus'
    @selected.css(
      'font-size': '',
      'background-color': '',
      'color': '',
    )
    @toolbar.find('.font').val 'f1'
    @edit.focus()

  toggleClass: (_class) ->
    @selected.toggleClass(_class)
    @edit.focus()

  deleteRow: ->
    @selected.closest('tr').remove()
    @body.find('tr th').each (i, el) ->
      $(el).text(i + 1)

  deleteColumn: ->
    x = parseInt(@selected.parent().data('x'))
    selector = 'tr td[data-x=' + x + ']'
    @body.find(selector).remove()

  initFastEditation: ->
    $(document).keydown (e) =>
      if e.which not in [37, 38, 39, 40, 9, 13]
        return

      if e.which == 13 # enter
        e = $.Event 'keydown'
        e.which = 40
        $(document).trigger e
        @recompute()
        return
      if e.which == 9 # tab
        e = $.Event 'keydown'
        e.which = 39
        $(document).trigger e
        @recompute()
        return

      x = parseInt @selected.parent().data('x')
      y = parseInt @selected.parent().data('y')
      switch e.which
        when 37
          x--
        when 38
          y--
        when 39
          x++
        when 40
          y++
      @selected.removeClass('focus')
      @selected = @body.find('td[data-x=' + x + '][data-y=' + y + '] input').addClass('focus')
      @edit.val(@selected.val()).focus()
      @recompute()

  recompute: ->
    @body.find('td input[value^="="]').each (i, el) ->
      el = $ el
      val = el.val().substring(1)
      # standalone number
      el.val(val)

  isValidCssColor: (color) ->
    isValid = /^#[0-9A-F]{6}$/i.test(color)
    if isValid
      return true

    if color in ['inherit', '', 'transparent']
      return false;

    image = document.createElement("img");
    image.style.color = "rgb(0, 0, 0)";
    image.style.color = color;
    if image.style.color != "rgb(0, 0, 0)"
      return true
    image.style.color = "rgb(255, 255, 255)"
    image.style.color = color;
    return image.style.color != "rgb(255, 255, 255)"

processor = new Processor '.root'
processor.init()

